#require "googleurlshortener/version"
require 'json'
require 'uri'
require 'net/http'

module Googleurlshortener

  class Default

    def shorten(long_url)
      uri = URI("https://www.googleapis.com/urlshortener/v1/url")
      body = ""
      Net::HTTP.start(uri.host, uri.port, :use_ssl => uri.scheme == 'https') do |http|
        request = Net::HTTP::Post.new uri.request_uri
        if defined? @auth_token && @auth_token != "" && @auth_token != nil
          request['Authorization'] = @auth_token
        end
        request["Content-Type"] = "application/json"
        response = http.request(request, "{\"longUrl\": \"#{long_url}\"}")
        body = response.body
      end
      return JSON.parse(body)["id"]
    end

    def expand(short_url)
      json = get_json_response("https://www.googleapis.com/urlshortener/v1/url?shortUrl=#{short_url}")
      return json["longUrl"]
    end
    
    private
    
      def get_json_response(url)
        uri = URI(url)
        request = Net::HTTP::Get.new(uri.request_uri)
        if self.class.name == "Googleurlshortener::Authenticated"
          request['Authorization'] = @auth_token
        end
        response = Net::HTTP.start(uri.hostname, uri.port, :use_ssl => uri.scheme == 'https') {|http|
          http.request(request)
        }
        return JSON.parse(response.body)
      end
  
  end
  
  class Authenticated < Default
    
    def initialize(email, password)
      @auth_token = get_authentication_token(email, password)
    end

    def analytics(short_url)
      return get_json_response("https://www.googleapis.com/urlshortener/v1/url?shortUrl=#{short_url}&projection=FULL")
    end
    
    private

      def get_authentication_token(email, password)
        uri = URI("https://www.google.com/accounts/ClientLogin")
        body = ""
        Net::HTTP.start(uri.host, uri.port, :use_ssl => uri.scheme == 'https') do |http|
          request = Net::HTTP::Post.new uri.request_uri
          data = "accountType=HOSTED_OR_GOOGLE&Email=#{email}&Passwd=#{password}&service=lh2&source=someapp1"
          response = http.request(request, data)
          body = response.body
        end
        start_index = body.index('Auth=')
        slice_of_auth_to_end = body[start_index..-1]
        end_index = slice_of_auth_to_end.index("\n")
        auth_string = slice_of_auth_to_end[0...end_index]
        auth_token = "GoogleLogin #{auth_string}"
        return auth_token
      end
    
  end

end
